function [nGoodImages] = createFeatureVectorMatchedToRange(dataset, imageIndex, makeMovieFlag, saveFilename, featureVectorType, weightFilename)
close all; 

%-----magic number to remove crappy laser data----
distanceStdThreshold = 3;

global bandWidth bandHeight thetaStep
global columnWidth nGridCol gridNum M N timeStampErrorThreshold

%-----parsing input arguements------
if nargin < 6
    weightFilename = '';
    viewPredictedDistanceFlag = false;
else
    viewPredictedDistanceFlag = true;
end
if nargin < 5
    featureVectorType = 'laws';
end
if nargin < 4
    saveFilename = '~/scratch/savedMatFiles/laws26.mat';
end
if nargin < 3
    makeMovieFlag = false;
end
if nargin < 2
    imageIndex = [3 1002];%[145 145];
end
if nargin < 1
    dataset = 'graphics10';%'laser1';
end
imageStart = imageIndex(1);
imageEnd = imageIndex(2);
%-------------------------------------------


%------Magic numbers for Laser calibration--------------
if strcmp(dataset, 'laser1')
%    indexPlot = 150:225;
    laserCenter = 182;
    laserLength = 76;   %length(indexPlot);
elseif strcmp(dataset, 'laser2')
%    indexPlot = 155:230;
    laserCenter = 192;
    laserLength = 76;  % length(indexPlot);
end
%----------------------------------------------------

%----creating paths and names--------
switch dataset
    case 'laser1'
	    imageDirectory = '../ImageData/';
    	timeSynchroError = 1;%1.3;
	    isGraphicsFlag = false;
    case 'laser2'
	    timeSynchroError = 1;%1.3;
	    imageDirectory = '~/scratch/ImageDataFeb9_small/';
	    isGraphicsFlag = false;
    case {'graphics1', 'graphics2', 'graphics3', 'graphics4', 'graphics5', 'graphics6', 'graphics7', 'graphics8', 'graphics9',
            'graphics10', 'graphics11', 'graphics12'}
	    gNumber = dataset(9);
	    imageDirectory = ['~jmichels/shared/newSimImages/' gNumber '/'];
	    isGraphicsFlag = true;
end
%filesList = dir(imageDirectory);
%save(['filesList' dataset '.mat'], 'filesList');
load(['filesList' dataset '.mat']);
%------------------------------------

%--------Movie making initialization--------
if makeMovieFlag
    if makeMovieFlag == 2
        mov = avifile('/tmp/Ashutosh/gates_1.avi');
        mov.fps = 2;
    end
    frameNumber = 0;
    fig = figure;
    set(fig,'DoubleBuffer','on');
    set(gca,'xlim',[1 176],'ylim',[1 144],...
       	   'NextPlot','replace','Visible','off')
end
%--------------------------------

%-------reading laser scan Data-------------
[rangeScanMatrix, timeStampRange] = parseRangeData(dataset);
rangeScanMatrix2 = medfilt2(rangeScanMatrix(:,:), [1 3]);
%-------------------------------------------

%--------reading Image file names---------------
filename = filesList(1000).name;
img = imread([imageDirectory filename]);
%-----------------------------------------------

%---------initializing matrices---------------
startMakingFeatureVector(img);
tmp = makeFeatureVector(img, featureVectorType, dataset);

maxFrames = imageEnd-imageStart+1;
featureVector = zeros( size(tmp,1), size(tmp,2), maxFrames);
timeStampMatchingError = 50*ones(1, maxFrames);
labeledDistance = -1*ones(nGridCol, maxFrames);
distanceVector = -1*ones(N, maxFrames);
%----------------------------------------------

arrayIndex = 1;
goodRangeScanFlag = true;
%-----going over each file one by one--------
for nFile = imageStart:imageEnd
    %---------matching nearest neighbour in Laser data--------
    filename = filesList(nFile).name;
    if strcmp(dataset, 'laser1') || strcmp(dataset, 'laser2')
        timeStampImage = double(sscanf( filename(9:14), '%d' ) + sscanf( filename(16:18), '%d' ) / 1000);
        [m closestIndex] = min( abs(timeStampRange - timeSynchroError - timeStampImage ));
        nRange = closestIndex;
        timeStampMatchingError(arrayIndex) = m;
    elseif isGraphicsFlag
	    timeStampImage = sscanf( filename(12:14), '%d' );
        timeStampRange;
	    [m closestIndex] = min( abs(timeStampRange - timeStampImage));
	    nRange = closestIndex;
	    timeStampMatchingError(arrayIndex) = m;
    end
    
    %----finding distance to nearest obstcale-------------
    if ~isGraphicsFlag
	    c = 1:N;
        %distanceVector2(:, arrayIndex) = rangeScanMatrix( nRange, round(laserCenter - (c-N/2)*laserLength/N) )';
    	   
    	distanceVector(:, arrayIndex) = rangeScanMatrix2( nRange, ...
        	round(laserCenter - (c-N/2)*laserLength/N) )';
    	for n = 1:nGridCol
       	    labeledDistance(n,arrayIndex) = min( distanceVector(n*columnWidth-(columnWidth-1):n*columnWidth, arrayIndex) );
        end
	    std = std(labeledDistance(:,arrayIndex));
        %sortedDistance = sort(labeledDistance(:,arrayIndex));
        %percentile10 = sortedDistance( round(0.1*length(sortedDistance)) );
        %percentile90 = sortedDistance( round(0.9*length(sortedDistance)) );        
        %percentileDiff = abs(percentile10-percentile90);
	    goodRangeScanFlag = std > distanceStdThreshold;
    elseif isGraphicsFlag
	    labeledDistance(:,arrayIndex) = rangeScanMatrix(nRange, :)';
	    goodRangeScanFlag = true;
    end
    
    %-----------reading image-----------------------
    if goodRangeScanFlag || makeMovieFlag || viewPredictedDistanceFlag
        switch featureVectorType
            case {'groundseg', 'groundseg1', 'groundSeg3', 'harris', 'none'}
            otherwise
               img = imread([imageDirectory filename]);

         end
        featureVector(:,:,arrayIndex) = makeFeatureVector(img, featureVectorType, dataset, timeStampImage);
    end

    if viewPredictedDistanceFlag
        [predictedRange, choosenDirection] = predictRange(weightFilename, featureVector(:,:,arrayIndex));
        %predictedRange = [20*ones(15,1); 50];
        %choosenDirection = 16;
        distanceVectorPredicted = predictedRange( ceil(nGridCol*(1:N)/N) );
        choosenXaxis = N*choosenDirection/nGridCol;
    end

    
    %------plotting to create movie---------
    if makeMovieFlag %&& timeStampMatchingError(arrayIndex) < timeStampErrorThreshold
        img = imresize(img, [M N], 'bilinear');
        
        image(img);    hold on;  
        plot( M - 1.5*distanceVector(:,arrayIndex), 'ro-'); 
                
        if ~goodRangeScanFlag
            plot( 72*ones(N,1), 'go-');      
        end
        if viewPredictedDistanceFlag
            'here'
            plot( M - 1.5*distanceVectorPredicted, 'bo-');
            fill( choosenXaxis(1) + [-5 -5 5 5], 50 + [-5 5 5 -5], 'y');
            fill( choosenXaxis(2) + [-4 -4 4 4], 75 + [-4 4 4 -4], 'y');
            fill( choosenXaxis(3) + [-3 -3 3 3], 100 + [-3 3 3 -3], 'y');
        end
        hold off; axis tight;
        
        text(10,10, ['Image: ' num2str(timeStampImage) '    Laser match = ' num2str(timeStampRange(nRange)) ...
            '    Image no. = ' num2str(nFile) '  timeMatchingError= ' num2str(timeStampMatchingError(arrayIndex)) ]);
        frameNumber = frameNumber + 1;
        mFrame = getframe(gca);
        if makeMovieFlag == 2
            mov = addframe(mov, mFrame);
        end
        %ginput;
    end
    %----------end creating movie-----------
    %arrayIndex = nFile - imageStart + 1;
    nFile - imageStart + 1;
    arrayIndex = arrayIndex + goodRangeScanFlag;
end

nGoodImages = arrayIndex - 1;
featureVector = featureVector( 1:size(featureVector,1), 1:size(featureVector,2), 1:nGoodImages);
labeledDistance = labeledDistance( 1:size(labeledDistance,1), 1:nGoodImages);
timeStampMatchingError = timeStampMatchingError( 1:size(timeStampMatchingError,1), 1:nGoodImages);

save(saveFilename, 'featureVector', 'labeledDistance', 'timeStampMatchingError');

if makeMovieFlag == 2
    mov = close(mov);
end
