function [featureVector] = makeFeatureTexture(img, type)
%this function returnd and displays feature vector from the image 
%startMakingNewFeatureTexture should be executed once before calling this
%function
persistent L3 E3 S3         % the Laws' masks

global nTopRadon thetaStep
global columnWidth nGridCol gridNum M N rowWidth

if nargin  < 2
    type = 'lawsradon';
end

if nargin < 1
    mov = aviread('manualOne.avi');
    img = mov(1,frame).cdata;
end

switch lower(type)
    case 'purelaws'
        nLaw = 9;
        nRadon = 0;
    case 'laws'
        nLaw = 11;
        nRadon = 0;
    case 'radon'
        nLaw = 0;
        nRadon = nTopRadon*180/thetaStep;
    case {'lawsradon', 'texture'}       % 'texture' means the best one, which right now is lawsradon
        nLaw = 11;
        nRadon = nTopRadon*180/thetaStep;
end
    
%convultional filters
if length(L3) <= 0
    L3 = [1 2 1];
    E3 = [-1 0 1];
    S3 = [-1 2 -1];
end

ycbcr_img = double(rgb2ycbcr(img)); %This computer has no image processing functions!!

%ycbcr_img(:,:,1) = ycbcr_img(:,:,1) - mean( mean( ycbcr_img(:,:,1) ));
%ycbcr_img(:,:,2) = ycbcr_img(:,:,2) - mean( mean( ycbcr_img(:,:,2) ));
%ycbcr_img(:,:,3) = ycbcr_img(:,:,3) - mean( mean( ycbcr_img(:,:,3) ));

imgY = ycbcr_img(:,:,1);% - mean( mean( ycbcr_img(:,:,1) ));
imgEdge = edge(imgY, 'sobel');

L3img = conv2(imgY, L3, 'same');
E3img = conv2(imgY, E3, 'same');
S3img = conv2(imgY, S3, 'same');
   
H(:,:,1) = conv2(L3img, L3', 'same');
H(:,:,2) = conv2(E3img, L3', 'same');
H(:,:,3) = conv2(S3img, L3', 'same');
   
H(:,:,4) = conv2(L3img, E3', 'same');
H(:,:,5) = conv2(E3img, E3', 'same');
H(:,:,6) = conv2(S3img, E3', 'same');
   
H(:,:,7) = conv2(L3img, S3', 'same');
H(:,:,8) = conv2(E3img, S3', 'same');
H(:,:,9) = conv2(S3img, S3', 'same');
   
H(:,:,10) = conv2( conv2( (ycbcr_img(:,:,2)), L3, 'same'), L3', 'same');
H(:,:,11) = conv2( conv2( (ycbcr_img(:,:,3)), L3, 'same'), L3', 'same');
  
H = abs(H);
for g = 1:gridNum
    for c = 1:nGridCol
        %LAWS-----------
        if nLaw
            featureVector2(g,c,1:nLaw) = mean( mean( H(g*rowWidth-(rowWidth-1):g*rowWidth+rowWidth, ...
            c*columnWidth-(columnWidth-1):c*columnWidth, 1:nLaw), 2), 1);
        end
        
        %RADON-------------------------
        if nRadon  
            temp = radon( imgEdge(g*rowWidth-(rowWidth-1):g*rowWidth, ...
                c*columnWidth-(columnWidth-1):c*columnWidth), 0:thetaStep:179 );
            temp2 = -sort(-temp,1);
            tempRadonFeature = reshape(temp2(1:nTopRadon,:), 1, nTopRadon*size(temp,2));
            if max(tempRadonFeature) > 0
                radonFeature = tempRadonFeature / max(tempRadonFeature);
            else
                radonFeature = zeros( size(tempRadonFeature));
            end
            featureVector2(g,c, (nLaw+1):(nLaw+nRadon) ) = permute(radonFeature, [1 3 2] );
        end
    end
end
   
featureVector3 = permute( featureVector2, [1 3 2]);
featureVector = reshape( featureVector3, [size(featureVector2,1)*size(featureVector3,2) nGridCol] );

%    if displayFlag
%        figure, 
%        subplot(2,2,1), imshow(img);
%        subplot(2,2,2), imagesc(ycbcr_img(:,:,1));
%        subplot(2,2,4), imagesc(ycbcr_img(:,:,2));
%        subplot(2,2,3); plot( squeeze(featureVector(frame,:,:)) );
%        axis tight; 
%    end
   
return;