function [featureVector] = makeFeatureVector(img, type, dataset, timeStampImage)

global nGridCol;

if nargin < 4
    timeStampImage = 0;
end
if nargin < 3
    dataset = 'laser1';
end

if nargin < 2
    type = 'texture';
end

global M N

switch lower(type)
    case {'rgb', 'gray', 'ycbcr', 'y'}
        imageResizeFactor = 0.5;
        if size(img,1) ~= M || size(img,2) ~= N
            img = imresize(img, [M N], 'bilinear');
        end
        featureVector = makeFeatureVectorGray(img, type, imageResizeFactor);
    case 'none'
        featureVector = zeros(0,nGridCol);
    case {'groundseg', 'groundseg3', 'groundseg1'}
	    featureVector = makeFeatureGroundSeg(type, timeStampImage, dataset);
    case 'harris'
        featureVector = makeFeatureVectorHarris(type, timeStampImage, dataset);
    case {'everything', 'everythingcar'}
        featureVector = makeFeatureVectorEverythingCar(type, timeStampImage, dataset);
    case {'lawsharrisgroundseg3'}
        featureVector1 = makeFeatureTexture(img, 'laws');
        featureVector2 = makeFeatureVectorHarris('harris', timeStampImage, dataset);
  	    featureVector3 = makeFeatureGroundSeg('groundseg3', timeStampImage, dataset);
        featureVector = [featureVector1; featureVector2; featureVector3];
    otherwise
        if size(img,1) ~= M || size(img,2) ~= N
            img = imresize(img, [M N], 'bilinear');
        end
        featureVector = makeFeatureTexture(img, type);
end

return;
