function [nDataPoints, nDimensions,  rawError, sortedAbsError, relativeError, sortedAbsRelativeError, ...
        rankError, sortedAbsRankError, ...
        choosenDirectionError, bestDistanceError, predictedDistanceHazardous, ...
        obstacleDirectionError, obstacleDistanceError, ...
    vectorChoosenBestColumn, fractionHazardousColumns ] = ...
    trainAndTest(whichData, trainOrTest, trainingType, filenameWeight, featureNames, fileNumbers, ...
        chooseFactor, groundSegVisionFlag, overlappingFlag, specialTmpFlag)

global nearestDistance largestDistance hazardDistance
global timeStampErrorThreshold directoryPath nGridCol

if nargin < 1
    whichData = 1; %1 or 2  %training Data by default
end
if nargin < 2
    trainOrTest = 'train';
end
if nargin < 3
    trainingType = 'linear';
end
if nargin < 4
    filenameWeight = '';
end
if nargin < 5
    featureNames = filenameWeight;
end
if nargin < 6
    fileNumbers = 1:16;
end
if nargin < 7
    chooseFactor = 3;
end
if nargin < 8
    groundSegVisionFlag = false;
end
if nargin < 9
    overlappingFlag = false;
end
if nargin < 10
    specialTmpFlag = false;
end

A = [];
b = [];
for nF = 1:length(featureNames)
    if iscellstr(featureNames)
        featureType = char(featureNames(nF));
    else
        featureType = featureNames;
    end
    [A1, b1, b_GroundSegAll] = loadLearningMatrices(fileNumbers, featureType, whichData, chooseFactor, overlappingFlag, specialTmpFlag);
    A = [A A1];
    b = b1;
end

if strcmp(trainOrTest, 'test') || strcmp(trainingType, 'linear')
    if ~specialTmpFlag
        A = [repmat(1,size(A,1),1) A];
    end
end

size(A);
size(b);

if strcmp(trainOrTest, 'train')
    if strcmp(trainingType, 'robust')
        disp(['Training Robustly on Matrix size = ' num2str(size(A,1)) 'x' num2str(size(A,2)) '.........']);
        w = robustfit(A, b, 'fair');      %cauchy performs slightly worse
        A = [repmat(1,size(A,1),1) A];
    elseif strcmp(trainingType, 'linear')
        disp(['Training Linearly on Matrix size = ' num2str(size(A,1)) 'x' num2str(size(A,2)) '.........']);
        if size(A,2) < 1100
            w = pinv(A)*b;
        else
            disp('Matrix too Large.....using approximate solution w = A\b');
            w = A \ b;
        end
    end
    if specialTmpFlag
        save([directoryPath 'tmptmptmp.mat'], 'w');
        'Saved to tmptmptmp.mat'
    else
        save([directoryPath 'weights_' filenameWeight '_' trainingType '.mat'], 'w');
    end
elseif strcmp(trainOrTest, 'test')
    if specialTmpFlag
        load([directoryPath 'tmptmptmp.mat']);
        'Loading weights from tmptmptmp'
    else
        load([directoryPath 'weights_' filenameWeight '_' trainingType '.mat']);
    end
    %if min(fileNumbers) <= 16
    %    GraphicsFeaturesChoose = [1:121 293:413 585:705 877];
    %    A = A(:,GraphicsFeaturesChoose);
    %    w = w([1:121 128:248 255:375 382]);
    %end
end
nDataPoints = size(A, 1);
nDimensions = size(A, 2);

%HACK FOR GRAPHICS
  
size(A);
size(w);
predictedDistance = A*w;
clippedDistance =  (predictedDistance < nearestDistance) .* nearestDistance + ...
                 (predictedDistance > nearestDistance & predictedDistance < largestDistance) .* predictedDistance + ...
                (predictedDistance > largestDistance) .* largestDistance;
size(clippedDistance);
            
if groundSegVisionFlag
    clippedDistance = b_GroundSegAll;
    'groundSegVisionFlag: using groundSeg directly'
end

%        %HACK HACK HACK
if specialTmpFlag  
    nGridCol = 14;
end
%nGridCol = 14;

%-----------absolute depth error-----------
rawError = clippedDistance-b;
rawError = rawError - mean(rawError);
sortedAbsError = -sort( -abs(rawError) );

%---------relative depth error-------------
pDistance = reshape(clippedDistance, nGridCol, length(clippedDistance)/nGridCol);
labeledDistance = reshape(b, nGridCol, length(b)/nGridCol);
e = pDistance-labeledDistance;
meanEachImage = mean(e, 1);
meanCorrectedError = e - repmat(meanEachImage, size(e,1), 1);

relativeError = reshape(meanCorrectedError, 1, size(meanCorrectedError,1)*size(meanCorrectedError,2) );
sortedAbsRelativeError = -sort( -abs(relativeError) );

%-------ranked Error metrics---------
pDistance = reshape(predictedDistance, nGridCol, length(clippedDistance)/nGridCol);

%----------choosen direction rank error------------
[tmplD ind] = sort(-labeledDistance, 1);
actualColumnRank = ind(1:4,:);
rankedOrder1 = zeros( size(labeledDistance) );
for k=1:size(labeledDistance, 2)
        rankedOrder1(ind(:,k), k) = (1:size(labeledDistance, 1))';
end
[tmp ind] = sort(-pDistance+1e-8*rand(size(pDistance)), 1);       % to break the tie randomly
for k=1:size(pDistance,2)
    actualRank(1,k) = rankedOrder1(ind(1,k), k);
    actualBestDistance(1,k) = -tmplD( 1, k);
    predictedBestColumnDistance(1,k) = labeledDistance( ind(1,k), k);
    vectorChoosenBestColumn(5:8,k) = labeledDistance( ind(1:4,k), k);
end

vectorChoosenBestColumn(1:4,:) = ind(1:4,:);
%vectorChoosenBestColumn(4,:) = predictedBestColumnDistance;

%vectorChoosenBestColumn(8,:) = actualBestDistance;
choosenDirectionError = -sort( -1./abs(actualRank) );
bestDistanceError = -sort( -abs( actualBestDistance - predictedBestColumnDistance) );

%predictedDistanceHazardous
predictedDistanceHazardous = predictedBestColumnDistance < log(hazardDistance);
actualBestColumnHazardous = actualBestDistance < log(hazardDistance);
actualWorstColumnNotHazardous = (-tmplD( size(tmplD,1),:)) > log(hazardDistance);
fractionHazardousColumns = mean(labeledDistance < log(hazardDistance), 1);
%----------choosen direction distance error-----


%----------sorted rank error---------------
[tmp ind] = sort(pDistance, 1);
predictedOrder = zeros( size(pDistance) );
for k=1:size(pDistance, 2)
    predictedOrder(ind(:,k), k) = (1:size(pDistance, 1))';
end
predictedOrderV = reshape(predictedOrder, 1, size(predictedOrder,1)*size(predictedOrder, 2) )';

[tmp ind] = sort(labeledDistance, 1);
rankedOrder1 = zeros( size(labeledDistance) );
for k=1:size(labeledDistance, 2)
        rankedOrder1(ind(:,k), k) = (1:size(labeledDistance, 1))';
end
rankedOrder = reshape(rankedOrder1, 1, size(labeledDistance,1)*size(labeledDistance,2) )';

rankError = rankedOrder - predictedOrderV;
sortedAbsRankError = -sort(-abs(rankError));
%-------------------------------------

%----------nearest Obstacle metric---------
%----------choosen direction rank error------------
[tmplD ind] = sort(labeledDistance, 1);
rankedOrder1 = zeros( size(labeledDistance) );
for k=1:size(labeledDistance, 2)
        rankedOrder1(ind(:,k), k) = (1:size(labeledDistance, 1))';
end
[tmp ind] = min(pDistance+1e-8*rand(size(pDistance)), [], 1);
for k=1:size(pDistance,2)
    actualRank(1,k) = rankedOrder1(ind(k), k);
    actualBestDistance(1,k) = tmplD( 1, k);
    predictedBestColumnDistance(1,k) = labeledDistance( ind(k), k);
end
obstacleDirectionError = -sort( -1./abs(actualRank) );
obstacleDistanceError = -sort( -abs( actualBestDistance - predictedBestColumnDistance) );

%----------choosen direction distance error-----


%--------------------------------------------


return;
