function [nDataPoints, nDimensions,  rawError, sortedAbsError, relativeError, sortedAbsRelativeError, rankError, sortedAbsRankError] = trainAndTest(whichData, trainOrTest, trainingType, filename)

global nearestDistance largestDistance
global timeStampErrorThreshold directoryPath

if nargin < 1
    whichData = 1; %1 or 2  %training Data by default
end
if nargin < 2
    trainOrTest = 'train';
end
if nargin < 3
    trainingType = 'linear';
end
if nargin < 4
    filename = '';
end

A = [ ];
b = [ ];
%-------------
for n=1:3%15 %[1 2 3]
    load([directoryPath filename num2str(n) '.mat']);
    clear distanceVector;
    
    goodTrainingSet = timeStampMatchingError < timeStampErrorThreshold;
    clear timeStampMatchingError;
    
    featureV = featureVector(:,:, find(goodTrainingSet) );
    clear featureVector;
    size(featureV);
    
    labeledD = labeledDistance(:, find(goodTrainingSet) );
    clear labeledDistance;
    
    if whichData == 1
        chooseImages = find( mod(1:size(labeledD,2),3) ~= 0);  %~=
    elseif whichData == 2
        chooseImages = find( mod(1:size(labeledD,2),3) == 0);
    end
    labeledD = labeledD(:, chooseImages);
    featureV = featureV(:,:, chooseImages);    
    
    b1 = log( reshape(labeledD, 1, size(labeledD,1)*size(labeledD,2)) )';
    
    A1 = reshape( featureV, size(featureV,1), size(featureV,2)*size(featureV,3))';
    clear featureV;
    clear labeledD;
        
    
    A = [A; A1];
    b = [b; b1];
end
clear b1 A1;

if strcmp(trainOrTest, 'test') || strcmp(trainingType, 'linear')
    A = [repmat(1,size(A,1),1) A];
end

if strcmp(trainOrTest, 'train')
    if strcmp(trainingType, 'robust')
        'training robustly'
        w = robustfit(A, b, 'fair');
        A = [repmat(1,size(A,1),1) A];
    elseif strcmp(trainingType, 'linear')
        'training linearly'
        w = pinv(A)*b;
    end
    save([filename trainingType '.mat'], 'w');
elseif strcmp(trainOrTest, 'test')
    load([filename trainingType '.mat']);
end
nDataPoints = size(A, 1)
nDimensions = size(A, 2)
size(w);

predictedDistance = A*w;
clippedDistance =  (predictedDistance < nearestDistance) .* nearestDistance + ...
                 (predictedDistance > nearestDistance & predictedDistance < largestDistance) .* predictedDistance + ...
                (predictedDistance > largestDistance) .* largestDistance;

%-----------absolute depth error-----------
rawError = clippedDistance-b;
sortedAbsError = -sort( -abs(rawError) );

%---------relative depth error-------------
pDistance = reshape(clippedDistance, 16, length(clippedDistance)/16);
labeledDistance = reshape(b, 16, length(b)/16);
e = pDistance-labeledDistance;
meanEachImage = mean(e, 1);
meanCorrectedError = e - repmat(meanEachImage, size(e,1), 1);

relativeError = reshape(meanCorrectedError, 1, size(meanCorrectedError,1)*size(meanCorrectedError,2) );
sortedAbsRelativeError = -sort( -abs(relativeError) );

%----------choosen direction error------------


%----------choosen direction distance error-----


%----------sorted rank error---------------
[tmp ind] = sort(pDistance, 1);
predictedOrder = zeros( size(pDistance) );
for k=1:size(pDistance, 2)
    predictedOrder(ind(:,k), k) = (1:size(pDistance, 1))';
end
predictedOrderV = reshape(predictedOrder, 1, size(predictedOrder,1)*size(predictedOrder, 2) )';

[tmp ind] = sort(labeledDistance, 1);
rankedOrder1 = zeros( size(labeledDistance) );
for k=1:size(labeledDistance, 2)
        rankedOrder1(ind(:,k), k) = (1:size(labeledDistance, 1))';
end
rankedOrder = reshape(rankedOrder1, 1, size(labeledDistance,1)*size(labeledDistance,2) )';

rankError = rankedOrder - predictedOrderV;
sortedAbsRankError = -sort(-abs(rankError));
%-------------------------------------


return;
