function [vectorBadDecision, vectorChoosenBestColumn, meanFractionHazardousColumns] = ...
    trainingCurve(featureType, trainingType, whichData, graphicsLevel, featureOverlappingFlag, displayFlag, specialTmpFlag)
global directoryPath
groundSegVisionFlag = false;

if nargin < 7
    specialTmpFlag = false;
end
if nargin < 6
    displayFlag = true;
end
if nargin < 5
    featureOverlappingFlag = false;
end
if nargin < 4
    graphicsLevel = 8;
end
if nargin < 3
    whichData = 'laser';
end
if nargin < 2
    trainingType = 'linear';            % robust or linear
end
if nargin < 1
    featureType = 'purelaws';           % none, purelaws, laws, radon, lawsradon, ycbcr, gray
end
specialFilename = '';%whichData;
if iscellstr(featureType)
    fT = [];
    for nF = 1:length(featureType)
        fT = [fT char(featureType(nF))];
    end
    featureFilename = [specialFilename fT];
else
    featureFilename = [specialFilename featureType];
end
%featureFilenameData = [dataFilename featureType];
isGraphicsFlag = false;
switch lower(whichData)
    case 'laser'
        if specialTmpFlag
            datasetsNumber = [1 2];
            fileNumbers = [1:10];%%[1:11 12 13:16];
        else
            datasetsNumber = [2];%[1 2];%[1 2];
            fileNumbers = 4:16;%[1:11 12 13:16];
        end
         chooseFactor = 3;
    case 'graphics'
        datasetsNumber = 3;
        fileNumbers = 16+graphicsLevel;
        isGraphicsFlag = true;
        chooseFactor = 3;
    case 'both'
        datasetsNumber = [1 2 3];
        fileNumbers = [1:11 12 13:16 16+graphicsLevel];
        chooseFactor = 2;
end

if specialTmpFlag
    testFileNumbers = [1:10];%11 12 13:16];
else
    testFileNumbers = 4:16;%[1:11 12 13:16];
end

trainDataNumber = 1;      %1 (take 2out of 3) or 2 (take every 3rd), or 3 (all)
testTrainDataNumber = 1; 
testTestDataNumber = 2;
if isGraphicsFlag
    testTrainDataNumber = 2;        % test on held out data of graphics
end

configurationFile(whichData);
arrayIndex = 0;
for arrayIndex=1:1
    %-----------forming Feature vectors--------------
    if ~iscellstr(featureType)
        createTrainingSet(featureType, datasetsNumber, specialFilename, graphicsLevel);      %1 means only data set 1, 2 means both data sets
    end

    %-----------------training-------------------
    if ~specialTmpFlag
        [nData dim] = trainAndTest(trainDataNumber, 'train', trainingType, featureFilename, featureType, ...
        fileNumbers, chooseFactor, groundSegVisionFlag, featureOverlappingFlag); 
    else
        outsideTraining(fileNumbers, 'everythingcar', 1, chooseFactor);
    end
      
    %-------------Error on TRAIN data------------
    [nDataPointsTrain, nDimensions, rawErrorTrain, sortedAbsErrorTrain, ...
       rawRelativeErrorTrain, sortedAbsRelativeErrorTrain, rawRankErrorTrain, sortedAbsRankErrorTrain,...
       rawChoosenDirectionErrorTrain, rawBestDistanceErrorTrain, ...
       rawPredictedDistanceHazardousTrain, ...
       rawObstacleDirectionErrorTrain, rawObstacleDistanceErrorTrain ] = ...
        trainAndTest(testTrainDataNumber, 'test', trainingType, featureFilename, featureType, ...
            fileNumbers, chooseFactor, groundSegVisionFlag, featureOverlappingFlag, specialTmpFlag); 
    errorTrain(arrayIndex) = mean( sortedAbsErrorTrain );
    sqErrorTrain(arrayIndex) = sqrt( mean(sortedAbsErrorTrain.^2) );
    relativeErrorTrain(arrayIndex) = mean( sortedAbsRelativeErrorTrain );
    relativeSqErrorTrain(arrayIndex) = sqrt( mean(sortedAbsRelativeErrorTrain.^2) );
    rankErrorTrain(arrayIndex) = mean( sortedAbsRankErrorTrain);
    choosenDirectionErrorTrain(arrayIndex) = mean(rawChoosenDirectionErrorTrain);
    bestDistanceErrorTrain(arrayIndex) = mean(rawBestDistanceErrorTrain);
    predictedDistanceHazardousTrain(arrayIndex) = sum(rawPredictedDistanceHazardousTrain)/ length(rawPredictedDistanceHazardousTrain);
    obstacleDirectionErrorTrain(arrayIndex) = mean(rawObstacleDirectionErrorTrain);
    obstacleDistanceErrorTrain(arrayIndex) = mean(rawObstacleDistanceErrorTrain);
    disp(['Train: Depth, r-Depth, all-rank, 1/N (^), bestDist, Hazard']);
    disp([errorTrain(arrayIndex)  relativeErrorTrain(arrayIndex) ...
            rankErrorTrain(arrayIndex) ...
            choosenDirectionErrorTrain(arrayIndex) bestDistanceErrorTrain(arrayIndex) ...
            predictedDistanceHazardousTrain(arrayIndex) ]);
    
    %----------Error on TEST data---------------------
    [nDataPointsTest, nDimensions, rawErrorTest, sortedAbsErrorTest, ...
            rawRelativeErrorTest, sortedAbsRelativeErrorTest, rawRankErrorTest, sortedAbsRankErrorTest,...
        rawChoosenDirectionErrorTest, rawBestDistanceErrorTest, ...
        rawPredictedDistanceHazardousTest, ...
        rawObstacleDirectionErrorTest, rawObstacleDistanceErrorTest, ...
        vectorChoosenBestColumn, fractionHazardousColumns ] = ...
        trainAndTest(testTestDataNumber, 'test', trainingType, featureFilename, featureType, ...
            testFileNumbers, chooseFactor, groundSegVisionFlag, featureOverlappingFlag, specialTmpFlag); 
    errorTest(arrayIndex) = mean( sortedAbsErrorTest );
    sqErrorTest(arrayIndex) = sqrt( mean(sortedAbsErrorTest.^2) );
    relativeErrorTest(arrayIndex) = mean( sortedAbsRelativeErrorTest );
    relativeSqErrorTest(arrayIndex) = sqrt( mean(sortedAbsRelativeErrorTest.^2) );
    rankErrorTest(arrayIndex) = mean(sortedAbsRankErrorTest);
    choosenDirectionErrorTest(arrayIndex) = mean(rawChoosenDirectionErrorTest);
    bestDistanceErrorTest(arrayIndex) = mean(rawBestDistanceErrorTest);
    predictedDistanceHazardousTest(arrayIndex) = sum(rawPredictedDistanceHazardousTest)/ length(rawPredictedDistanceHazardousTest);
    obstacleDirectionErrorTest(arrayIndex) = mean(rawObstacleDirectionErrorTest);
    obstacleDistanceErrorTest(arrayIndex) = mean(rawObstacleDistanceErrorTest);
    
    vectorBadDecision = rawPredictedDistanceHazardousTest;
    meanFractionHazardousColumns = mean(fractionHazardousColumns);
    
    disp(['Test: Depth, r-Depth, all-rank, 1/N (^), bestDist, Hazard']);
    disp([errorTest(arrayIndex) relativeErrorTest(arrayIndex) ...
            rankErrorTest(arrayIndex) ...
            choosenDirectionErrorTest(arrayIndex) bestDistanceErrorTest(arrayIndex) ...
            predictedDistanceHazardousTest(arrayIndex) ]);
    
    disp(['nDimensions = ' num2str(nDimensions) ',     nTrainPoints = ' num2str(nDataPointsTrain) ...
            ',  nTestPoints = ' num2str(nDataPointsTest) ',   Training = ' trainingType ',  Feature = ' featureFilename]);
    
    save([directoryPath 'results' featureFilename '_' trainingType '_' whichData '.mat'], 'errorTrain', 'errorTest', 'sqErrorTrain', 'sqErrorTest', ...
        'relativeErrorTrain', 'relativeSqErrorTrain', 'relativeErrorTest', 'relativeSqErrorTest', 'rankErrorTrain', 'rankErrorTest', ...
        'choosenDirectionErrorTrain', 'bestDistanceErrorTrain', 'choosenDirectionErrorTest', 'bestDistanceErrorTest', ...
        'nDataPointsTrain', 'nDataPointsTest', 'nDimensions');
  
    
    if displayFlag
        figure,
        subplot(6,2,1); plot(sortedAbsErrorTrain); title('Train'); xlabel('Depth');
        subplot(6,2,2); plot(sortedAbsErrorTest); title('Test');
        subplot(6,2,3); plot(sortedAbsRelativeErrorTrain); xlabel('Rel Depth');
        subplot(6,2,4); plot(sortedAbsRelativeErrorTest); title(featureFilename);
        subplot(6,2,5); plot(sortedAbsRankErrorTrain); xlabel('Sorted Rank');
        subplot(6,2,6); plot(sortedAbsRankErrorTest);
        subplot(6,2,7); plot(rawChoosenDirectionErrorTrain); xlabel('theta');
        subplot(6,2,8); plot(rawChoosenDirectionErrorTest);
        subplot(6,2,9); plot(rawBestDistanceErrorTrain); xlabel('Distance in theta');
        subplot(6,2,10); plot(rawBestDistanceErrorTest);
        subplot(6,2,11); plot(rawPredictedDistanceHazardousTrain); xlabel('Hazard');
        subplot(6,2,12); plot(rawPredictedDistanceHazardousTest);
        %subplot(8,2,13); plot(rawObstacleDirectionErrorTrain);
        %subplot(8,2,14); plot(rawObstacleDirectionErrorTest);
        %subplot(8,2,15); plot(rawObstacleDistanceErrorTrain);
        %subplot(8,2,16); plot(rawObstacleDistanceErrorTest);
        %figure(100),
        %subplot(2,2, strcmp(whichData, 'laser')+1); plot(vectorBadDecision); title(whichData);
        %subplot(3,2, 2+strcmp(whichData, 'laser')+1); plot(vectorAllHazardousColumns);
        %subplot(2,2, 2+strcmp(whichData, 'laser')+1); plot(vectorAllNonHazardousColumns);
    end
end

if displayFlag && arrayIndex > 1
    figure, 
    plot(errorTrain); hold on,
    plot(errorTest); hold off;

    figure,
    plot(sqErrorTrain); hold on;
    plot(sqErrorTest); hold off;
end
